@@ -13,9 +13,9 @@ export tunneling
 export qft
 
 @doc raw"""
-    rand_unitary(dimensions, distribution=Val(:haar))
+    rand_unitary([T::Type=ComplexF64,] dimensions, distribution=Val(:haar))
 
-Returns a random unitary [`QuantumObject`](@ref).
+Returns a random unitary [`QuantumObject`](@ref) with element type `T = ComplexF64` (default).
 
 The `dimensions` can be either the following types:
 - `dimensions::Int`: Number of basis states in the Hilbert space.
@@ -31,15 +31,15 @@ The `distribution` specifies which of the method used to obtain the unitary matr
 !!! warning "Beware of type-stability!"
     If you want to keep type stability, it is recommended to use `rand_unitary(dimensions, Val(distribution))` instead of `rand_unitary(dimensions, distribution)`. Also, put `dimensions` as `Tuple` or `SVector` from [StaticArrays.jl](https://github.com/JuliaArrays/StaticArrays.jl). See [this link](https://docs.julialang.org/en/v1/manual/performance-tips/#man-performance-value-type) and the [related Section](@ref doc:Type-Stability) about type stability for more details.
 """
-rand_unitary(dimensions::Int, distribution::Union{Symbol, Val} = Val(:haar)) =
-    rand_unitary(SVector(dimensions), makeVal(distribution))
-rand_unitary(dimensions::Union{Dimensions, AbstractVector{Int}, Tuple}, distribution::Union{Symbol, Val} = Val(:haar)) =
-    rand_unitary(dimensions, makeVal(distribution))
-function rand_unitary(dimensions::Union{Dimensions, AbstractVector{Int}, Tuple}, ::Val{:haar})
+rand_unitary(::Type{T}, dimensions::Int, distribution::Union{Symbol, Val} = Val(:haar)) where {T <: FloatOrComplex} =
+    rand_unitary(T, SVector(dimensions), makeVal(distribution))
+rand_unitary(::Type{T}, dimensions::Union{Dimensions, AbstractVector{Int}, Tuple}, distribution::Union{Symbol, Val} = Val(:haar)) where {T <: FloatOrComplex} =
+    rand_unitary(T, dimensions, makeVal(distribution))
+function rand_unitary(::Type{T}, dimensions::Union{Dimensions, AbstractVector{Int}, Tuple}, ::Val{:haar}) where {T <: FloatOrComplex}
     N = prod(dimensions)
 
     # generate N x N matrix Z of complex standard normal random variates
-    Z = randn(ComplexF64, N, N)
+    Z = randn(T, N, N)
 
     # find QR decomposition: Z = Q ⋅ R
     Q, R = LinearAlgebra.qr(Z)
@@ -50,19 +50,24 @@ function rand_unitary(dimensions::Union{Dimensions, AbstractVector{Int}, Tuple},
     Λ ./= abs.(Λ) # rescaling the elements
     return QuantumObject(to_dense(Q * Diagonal(Λ)); type = Operator(), dims = dimensions)
 end
-function rand_unitary(dimensions::Union{Dimensions, AbstractVector{Int}, Tuple}, ::Val{:exp})
+function rand_unitary(::Type{T}, dimensions::Union{Dimensions, AbstractVector{Int}, Tuple}, ::Val{:exp}) where {T <: FloatOrComplex}
     N = prod(dimensions)
 
     # generate N x N matrix Z of complex standard normal random variates
-    Z = randn(ComplexF64, N, N)
+    Z = randn(T, N, N)
 
     # generate Hermitian matrix
+    # make it a Qobj first, cause we need Qobj-ver. of exp() for special case in _rand_unitary_exp later
     H = QuantumObject((Z + Z') / 2; type = Operator(), dims = dimensions)
-
-    return to_dense(exp(-1.0im * H))
+    return to_dense(_rand_unitary_exp(H))
 end
-rand_unitary(dimensions::Union{Dimensions, AbstractVector{Int}, Tuple}, ::Val{T}) where {T} =
-    throw(ArgumentError("Invalid distribution: $(T)"))
+rand_unitary(::Type{T}, dimensions::Union{Dimensions, AbstractVector{Int}, Tuple}, ::Val{Td}) where {T <: FloatOrComplex, Td} =
+    throw(ArgumentError("Invalid distribution: $(Td)"))
+rand_unitary(dimensions::Union{Int, Dimensions, AbstractVector{Int}, Tuple}, distribution::Union{Symbol, Val} = Val(:haar)) = rand_unitary(ComplexF64, dimensions, distribution)
+
+# we make H sparse here because the following method currently does not exist : exp(::Matrix{Complex{BigFloat}})
+_rand_unitary_exp(H::QuantumObject{Operator, <:AbstractDimensions, Matrix{Complex{BigFloat}}}) = exp(-im * to_sparse(H))
+_rand_unitary_exp(H::QuantumObject{Operator, <:AbstractDimensions, <:AbstractArray}) = exp(-im * H)
 
 @doc raw"""
     commutator(A::QuantumObject, B::QuantumObject; anti::Bool=false)
@@ -76,9 +81,9 @@ Note that `A` and `B` must be [`Operator`](@ref)
 commutator(A::QuantumObject{Operator}, B::QuantumObject{Operator}; anti::Bool = false) = A * B - (-1)^anti * B * A
 
 @doc raw"""
-    destroy(N::Int)
+    destroy([T::Type=ComplexF64,] N::Int)
 
-Bosonic annihilation operator with Hilbert space cutoff `N`. 
+Bosonic annihilation operator with Hilbert space cutoff `N` and element type `T = ComplexF64` (default).
 
 This operator acts on a fock state as ``\hat{a} \ket{n} = \sqrt{n} \ket{n-1}``.
 
@@ -99,12 +104,13 @@ julia> fock(20, 3)' * a * fock(20, 4)
 2.0 + 0.0im
 ```
 """
-destroy(N::Int) = QuantumObject(spdiagm(1 => Array{ComplexF64}(sqrt.(1:(N - 1)))), Operator(), N)
+destroy(::Type{T}, N::Int) where {T <: FloatOrComplex} = QuantumObject(spdiagm(1 => T[sqrt(T(val)) for val in 1:(N - 1)]), Operator(), N)
+destroy(N::Int) = destroy(ComplexF64, N)
 
 @doc raw"""
-    create(N::Int)
+    create([T::Type=ComplexF64,] N::Int)
 
-Bosonic creation operator with Hilbert space cutoff `N`.
+Bosonic creation operator with Hilbert space cutoff `N` and element type `T = ComplexF64` (default).
 
 This operator acts on a fock state as ``\hat{a}^\dagger \ket{n} = \sqrt{n+1} \ket{n+1}``.
 
@@ -125,12 +131,13 @@ julia> fock(20, 4)' * a_d * fock(20, 3)
 2.0 + 0.0im
 ```
 """
-create(N::Int) = QuantumObject(spdiagm(-1 => Array{ComplexF64}(sqrt.(1:(N - 1)))), Operator(), N)
+create(::Type{T}, N::Int) where {T <: FloatOrComplex} = QuantumObject(spdiagm(-1 => T[sqrt(T(val)) for val in 1:(N - 1)]), Operator(), N)
+create(N::Int) = create(ComplexF64, N)
 
 @doc raw"""
     displace(N::Int, α::Number)
 
-Generate a [displacement operator](https://en.wikipedia.org/wiki/Displacement_operator):
+Generate a [displacement operator](https://en.wikipedia.org/wiki/Displacement_operator) with element precision same as `α`:
 
 ```math
 \hat{D}(\alpha)=\exp\left( \alpha \hat{a}^\dagger - \alpha^* \hat{a} \right),
@@ -139,14 +146,14 @@ Generate a [displacement operator](https://en.wikipedia.org/wiki/Displacement_op
 where ``\hat{a}`` is the bosonic annihilation operator, and ``\alpha`` is the amount of displacement in optical phase space.
 """
 function displace(N::Int, α::T) where {T <: Number}
-    a = destroy(N)
-    return exp(α * a' - α' * a)
+    a = destroy(_float_type(T), N) # use float type to handle integer α
+    return exp(α * a' - conj(α) * a) # the return eltype will be promoted to complex if α is complex
 end
 
 @doc raw"""
     squeeze(N::Int, z::Number)
 
-Generate a single-mode [squeeze operator](https://en.wikipedia.org/wiki/Squeeze_operator):
+Generate a single-mode [squeeze operator](https://en.wikipedia.org/wiki/Squeeze_operator) with element precision same as `z`:
 
 ```math
 \hat{S}(z)=\exp\left( \frac{1}{2} (z^* \hat{a}^2 - z(\hat{a}^\dagger)^2) \right),
@@ -155,47 +162,50 @@ Generate a single-mode [squeeze operator](https://en.wikipedia.org/wiki/Squeeze_
 where ``\hat{a}`` is the bosonic annihilation operator.
 """
 function squeeze(N::Int, z::T) where {T <: Number}
-    a_sq = destroy(N)^2
-    return exp((z' * a_sq - z * a_sq') / 2)
+    a_sq = destroy(_float_type(T), N)^2 # use float type to handle integer z
+    return exp((conj(z) * a_sq - z * a_sq') / 2) # the return eltype will be promoted to complex if z is complex
 end
 
 @doc raw"""
-    num(N::Int)
+    num([T::Type=ComplexF64,] N::Int)
 
-Bosonic number operator with Hilbert space cutoff `N`. 
+Bosonic number operator with Hilbert space cutoff `N` and element type `T = ComplexF64` (default). 
 
 This operator is defined as ``\hat{N}=\hat{a}^\dagger \hat{a}``, where ``\hat{a}`` is the bosonic annihilation operator.
 """
-num(N::Int) = QuantumObject(spdiagm(0 => Array{ComplexF64}(0:(N - 1))), Operator(), N)
+num(::Type{T}, N::Int) where {T <: Number} = QuantumObject(spdiagm(0 => Array{T}(0:(N - 1))), Operator(), N)
+num(N::Int) = num(ComplexF64, N)
 
 @doc raw"""
-    position(N::Int)
+    position([T::Type=ComplexF64,] N::Int)
 
-Position operator with Hilbert space cutoff `N`. 
+Position operator with Hilbert space cutoff `N` and element type `T = ComplexF64` (default).
 
 This operator is defined as ``\hat{x}=\frac{1}{\sqrt{2}} (\hat{a}^\dagger + \hat{a})``, where ``\hat{a}`` is the bosonic annihilation operator.
 """
-function position(N::Int)
-    a = destroy(N)
-    return (a' + a) / sqrt(2)
+function position(::Type{T}, N::Int) where {T <: FloatOrComplex}
+    a = destroy(T, N)
+    return (a' + a) / sqrt(T(2))
 end
+position(N::Int) = position(ComplexF64, N)
 
 @doc raw"""
-    momentum(N::Int)
+    momentum([T::Type=ComplexF64,] N::Int)
 
-Momentum operator with Hilbert space cutoff `N`. 
+Momentum operator with Hilbert space cutoff `N` and element type `T = ComplexF64` (default).
 
 This operator is defined as ``\hat{p}= \frac{i}{\sqrt{2}} (\hat{a}^\dagger - \hat{a})``, where ``\hat{a}`` is the bosonic annihilation operator.
 """
-function momentum(N::Int)
-    a = destroy(N)
-    return (1.0im * sqrt(0.5)) * (a' - a)
+function momentum(::Type{T}, N::Int) where {T <: Complex}
+    a = destroy(T, N)
+    return (a - a') / (im * sqrt(T(2)))
 end
+momentum(N::Int) = momentum(ComplexF64, N)
 
 @doc raw"""
     phase(N::Int, ϕ0::Real=0)
 
-Single-mode Pegg-Barnett phase operator with Hilbert space cutoff ``N`` and the reference phase ``\phi_0``.
+Single-mode Pegg-Barnett phase operator with Hilbert space cutoff ``N``, the reference phase ``\phi_0``, and element precision same as `ϕ0`.
 
 This operator is defined as
 
@@ -218,17 +228,17 @@ and
 # Reference
 - [Michael Martin Nieto, QUANTUM PHASE AND QUANTUM PHASE OPERATORS: Some Physics and Some History, arXiv:hep-th/9304036](https://arxiv.org/abs/hep-th/9304036), Equation (30-32).
 """
-function phase(N::Int, ϕ0::Real = 0)
+function phase(N::Int, ϕ0::T = 0) where {T <: Real}
     N_list = collect(0:(N - 1))
-    ϕ = ϕ0 .+ (2 * π / N) .* N_list
-    states = [exp.((1.0im * ϕ[m]) .* N_list) ./ sqrt(N) for m in 1:N]
+    ϕ = ϕ0 .+ (2 * _float_type(T)(π) / N) .* N_list # _float_type(T) can deal with T = Int and BigFloat
+    states = [exp.(im * ϕ[m] .* N_list) ./ sqrt(T(N)) for m in 1:N] # here promotes element type to complex
     return QuantumObject(sum([ϕ[m] * states[m] * states[m]' for m in 1:N]); type = Operator(), dims = N)
 end
 
 @doc raw"""
-    jmat(j::Real, which::Union{Symbol,Val})
+    jmat([T::Type=ComplexF64,] j::Real, which::Union{Symbol,Val})
 
-Generate higher-order Spin-`j` operators, where `j` is the spin quantum number and can be a non-negative integer or half-integer
+Generate higher-order Spin-`j` operators with element type `T = ComplexF64` (default). `j` is the spin quantum number and can be a non-negative integer or half-integer.
 
 The parameter `which` specifies which of the following operator to return.
 - `:x`: ``\hat{S}_x``
@@ -268,158 +278,174 @@ Quantum Object:   type=Operator()   dims=[4]   size=(4, 4)   ishermitian=true
 !!! warning "Beware of type-stability!"
     If you want to keep type stability, it is recommended to use `jmat(j, Val(which))` instead of `jmat(j, which)`. See [this link](https://docs.julialang.org/en/v1/manual/performance-tips/#man-performance-value-type) and the [related Section](@ref doc:Type-Stability) about type stability for more details.
 """
-jmat(j::Real, which::Symbol) = jmat(j, Val(which))
-jmat(j::Real) = (jmat(j, Val(:x)), jmat(j, Val(:y)), jmat(j, Val(:z)))
-function jmat(j::Real, ::Val{:x})
+jmat(::Type{T}, j::Real, which::Symbol) where {T <: FloatOrComplex} = jmat(T, j, Val(which))
+jmat(::Type{T}, j::Real) where {T <: FloatOrComplex} = (jmat(T, j, Val(:x)), jmat(T, j, Val(:y)), jmat(T, j, Val(:z)))
+function jmat(::Type{T}, j::Real, ::Val{:x}) where {T <: FloatOrComplex}
     J = 2 * j + 1
     ((floor(J) != J) || (j < 0)) &&
         throw(ArgumentError("The spin quantum number (j) must be a non-negative integer or half-integer."))
 
-    σ = _jm(j)
+    σ = _jm(T, j)
     return QuantumObject((σ' + σ) / 2, Operator(), Int(J))
 end
-function jmat(j::Real, ::Val{:y})
+function jmat(::Type{T}, j::Real, ::Val{:y}) where {T <: FloatOrComplex}
     J = 2 * j + 1
     ((floor(J) != J) || (j < 0)) &&
         throw(ArgumentError("The spin quantum number (j) must be a non-negative integer or half-integer."))
 
-    σ = _jm(j)
+    σ = _jm(T, j)
     return QuantumObject((σ' - σ) / 2im, Operator(), Int(J))
 end
-function jmat(j::Real, ::Val{:z})
+function jmat(::Type{T}, j::Real, ::Val{:z}) where {T <: FloatOrComplex}
     J = 2 * j + 1
     ((floor(J) != J) || (j < 0)) &&
         throw(ArgumentError("The spin quantum number (j) must be a non-negative integer or half-integer."))
 
-    return QuantumObject(_jz(j), Operator(), Int(J))
+    return QuantumObject(_jz(T, j), Operator(), Int(J))
 end
-function jmat(j::Real, ::Val{:+})
+function jmat(::Type{T}, j::Real, ::Val{:+}) where {T <: FloatOrComplex}
     J = 2 * j + 1
     ((floor(J) != J) || (j < 0)) &&
         throw(ArgumentError("The spin quantum number (j) must be a non-negative integer or half-integer."))
 
-    return QuantumObject(adjoint(_jm(j)), Operator(), Int(J))
+    return QuantumObject(adjoint(_jm(T, j)), Operator(), Int(J))
 end
-function jmat(j::Real, ::Val{:-})
+function jmat(::Type{T}, j::Real, ::Val{:-}) where {T <: FloatOrComplex}
     J = 2 * j + 1
     ((floor(J) != J) || (j < 0)) &&
         throw(ArgumentError("The spin quantum number (j) must be a non-negative integer or half-integer."))
 
-    return QuantumObject(_jm(j), Operator(), Int(J))
+    return QuantumObject(_jm(T, j), Operator(), Int(J))
 end
-jmat(j::Real, ::Val{T}) where {T} = throw(ArgumentError("Invalid spin operator: $(T)"))
+jmat(::Type{T}, j::Real, ::Val{T2}) where {T <: FloatOrComplex, T2} = throw(ArgumentError("Invalid spin operator: $(T2)"))
+jmat(j::Real, which::Union{Symbol, Val}) = jmat(ComplexF64, j, which)
+jmat(j::Real) = jmat(ComplexF64, j)
 
-function _jm(j::Real)
-    m = j:(-1):(-j)
-    data = sqrt.(j * (j + 1) .- m .* (m .- 1))[1:(end - 1)]
-    return spdiagm(-1 => Array{ComplexF64}(data))
+function _jm(::Type{T}, j::Real) where {T <: FloatOrComplex}
+    m = T.(j:(-1):(-j))
+    data = @. sqrt(T(j * (j + 1)) - m * (m - 1))
+    return spdiagm(-1 => data[1:(end - 1)])
+end
+function _jz(::Type{T}, j::Real) where {T <: FloatOrComplex}
+    data = @. T(j) - (0:Int(2 * j))
+    return spdiagm(0 => data)
 end
-_jz(j::Real) = spdiagm(0 => Array{ComplexF64}(j .- (0:Int(2 * j))))
 
 @doc raw"""
-    spin_Jx(j::Real)
+    spin_Jx([T::Type=ComplexF64,] j::Real)
 
-``\hat{S}_x`` operator for Spin-`j`, where `j` is the spin quantum number and can be a non-negative integer or half-integer.
+``\hat{S}_x`` operator for Spin-`j` with element type `T = ComplexF64` (default). `j` is the spin quantum number and can be a non-negative integer or half-integer.
 
 See also [`jmat`](@ref).
 """
-spin_Jx(j::Real) = jmat(j, Val(:x))
+spin_Jx(::Type{T}, j::Real) where {T <: FloatOrComplex} = jmat(T, j, Val(:x))
+spin_Jx(j::Real) = spin_Jx(ComplexF64, j)
 
 @doc raw"""
-    spin_Jy(j::Real)
+    spin_Jy([T::Type=ComplexF64,] j::Real)
 
-``\hat{S}_y`` operator for Spin-`j`, where `j` is the spin quantum number and can be a non-negative integer or half-integer.
+``\hat{S}_y`` operator for Spin-`j` with element type `T = ComplexF64` (default). `j` is the spin quantum number and can be a non-negative integer or half-integer.
 
 See also [`jmat`](@ref).
 """
-spin_Jy(j::Real) = jmat(j, Val(:y))
+spin_Jy(::Type{T}, j::Real) where {T <: FloatOrComplex} = jmat(T, j, Val(:y))
+spin_Jy(j::Real) = spin_Jy(ComplexF64, j)
 
 @doc raw"""
-    spin_Jz(j::Real)
+    spin_Jz([T::Type=ComplexF64,] j::Real)
 
-``\hat{S}_z`` operator for Spin-`j`, where `j` is the spin quantum number and can be a non-negative integer or half-integer.
+``\hat{S}_z`` operator for Spin-`j` with element type `T = ComplexF64` (default). `j` is the spin quantum number and can be a non-negative integer or half-integer.
 
 See also [`jmat`](@ref).
 """
-spin_Jz(j::Real) = jmat(j, Val(:z))
+spin_Jz(::Type{T}, j::Real) where {T <: FloatOrComplex} = jmat(T, j, Val(:z))
+spin_Jz(j::Real) = spin_Jz(ComplexF64, j)
 
 @doc raw"""
-    spin_Jm(j::Real)
+    spin_Jm([T::Type=ComplexF64,] j::Real)
 
-``\hat{S}_-`` operator for Spin-`j`, where `j` is the spin quantum number and can be a non-negative integer or half-integer.
+``\hat{S}_-`` operator for Spin-`j` with element type `T = ComplexF64` (default). `j` is the spin quantum number and can be a non-negative integer or half-integer.
 
 See also [`jmat`](@ref).
 """
-spin_Jm(j::Real) = jmat(j, Val(:-))
+spin_Jm(::Type{T}, j::Real) where {T <: FloatOrComplex} = jmat(T, j, Val(:-))
+spin_Jm(j::Real) = spin_Jm(ComplexF64, j)
 
 @doc raw"""
-    spin_Jp(j::Real)
+    spin_Jp([T::Type=ComplexF64,] j::Real)
 
-``\hat{S}_+`` operator for Spin-`j`, where `j` is the spin quantum number and can be a non-negative integer or half-integer.
+``\hat{S}_+`` operator for Spin-`j` with element type `T = ComplexF64` (default). `j` is the spin quantum number and can be a non-negative integer or half-integer.
 
 See also [`jmat`](@ref).
 """
-spin_Jp(j::Real) = jmat(j, Val(:+))
+spin_Jp(::Type{T}, j::Real) where {T <: FloatOrComplex} = jmat(T, j, Val(:+))
+spin_Jp(j::Real) = spin_Jp(ComplexF64, j)
 
 @doc raw"""
-    spin_J_set(j::Real)
+    spin_J_set([T::Type=ComplexF64,] j::Real)
 
-A set of Spin-`j` operators ``(\hat{S}_x, \hat{S}_y, \hat{S}_z)``, where `j` is the spin quantum number and can be a non-negative integer or half-integer.
+A set of Spin-`j` operators ``(\hat{S}_x, \hat{S}_y, \hat{S}_z)`` with element type `T = ComplexF64` (default). `j` is the spin quantum number and can be a non-negative integer or half-integer.
 
 Note that this functions is same as `jmat(j)`. See also [`jmat`](@ref).
 """
-spin_J_set(j::Real) = jmat(j)
+spin_J_set(::Type{T}, j::Real) where {T <: FloatOrComplex} = jmat(T, j)
+spin_J_set(j::Real) = spin_J_set(ComplexF64, j)
 
 @doc raw"""
-    sigmap()
+    sigmap([T::Type=ComplexF64])
 
-Pauli ladder operator ``\hat{\sigma}_+ = (\hat{\sigma}_x + i \hat{\sigma}_y) / 2``.
+Pauli ladder operator ``\hat{\sigma}_+ = (\hat{\sigma}_x + i \hat{\sigma}_y) / 2`` with element type `T = ComplexF64` (default).
 
 See also [`jmat`](@ref).
 """
-sigmap() = jmat(0.5, Val(:+))
+sigmap(::Type{T}) where {T <: FloatOrComplex} = jmat(T, 0.5, Val(:+))
+sigmap() = sigmap(ComplexF64)
 
 @doc raw"""
-    sigmam()
+    sigmam([T::Type=ComplexF64])
 
-Pauli ladder operator ``\hat{\sigma}_- = (\hat{\sigma}_x - i \hat{\sigma}_y) / 2``.
+Pauli ladder operator ``\hat{\sigma}_- = (\hat{\sigma}_x - i \hat{\sigma}_y) / 2`` with element type `T = ComplexF64` (default).
 
 See also [`jmat`](@ref).
 """
-sigmam() = jmat(0.5, Val(:-))
+sigmam(::Type{T}) where {T <: FloatOrComplex} = jmat(T, 0.5, Val(:-))
+sigmam() = sigmam(ComplexF64)
 
 @doc raw"""
-    sigmax()
+    sigmax([T::Type=ComplexF64])
 
-Pauli operator ``\hat{\sigma}_x = \hat{\sigma}_- + \hat{\sigma}_+``.
+Pauli operator ``\hat{\sigma}_x = \hat{\sigma}_- + \hat{\sigma}_+`` with element type `T = ComplexF64` (default).
 
 See also [`jmat`](@ref).
 """
-sigmax() = rmul!(jmat(0.5, Val(:x)), 2)
+sigmax(::Type{T}) where {T <: FloatOrComplex} = rmul!(jmat(T, 0.5, Val(:x)), 2)
+sigmax() = sigmax(ComplexF64)
 
 @doc raw"""
-    sigmay()
+    sigmay([T::Type=ComplexF64])
 
-Pauli operator ``\hat{\sigma}_y = i \left( \hat{\sigma}_- - \hat{\sigma}_+ \right)``.
+Pauli operator ``\hat{\sigma}_y = i \left( \hat{\sigma}_- - \hat{\sigma}_+ \right)`` with element type `T = ComplexF64` (default).
 
 See also [`jmat`](@ref).
 """
-sigmay() = rmul!(jmat(0.5, Val(:y)), 2)
+sigmay(::Type{T}) where {T <: FloatOrComplex} = rmul!(jmat(T, 0.5, Val(:y)), 2)
+sigmay() = sigmay(ComplexF64)
 
 @doc raw"""
-    sigmaz()
+    sigmaz([T::Type=ComplexF64])
 
-Pauli operator ``\hat{\sigma}_z = \left[ \hat{\sigma}_+ , \hat{\sigma}_- \right]``.
+Pauli operator ``\hat{\sigma}_z = \left[ \hat{\sigma}_+ , \hat{\sigma}_- \right]`` with element type `T = ComplexF64` (default).
 
 See also [`jmat`](@ref).
 """
-sigmaz() = rmul!(jmat(0.5, Val(:z)), 2)
+sigmaz(::Type{T}) where {T <: FloatOrComplex} = rmul!(jmat(T, 0.5, Val(:z)), 2)
+sigmaz() = sigmaz(ComplexF64)
 
 @doc raw"""
-    eye(N::Int; type=Operator, dims=nothing)
-    qeye(N::Int; type=Operator, dims=nothing)
+    eye([T::Type=ComplexF64,] N::Int; type=Operator, dims=nothing)
+    qeye([T::Type=ComplexF64,] N::Int; type=Operator, dims=nothing)
 
-Identity operator ``\hat{\mathbb{1}}`` with size `N`.
+Identity operator ``\hat{\mathbb{1}}`` with size `N` and element type `T = ComplexF64` (default).
 
 It is also possible to specify the list of Hilbert dimensions `dims` if different subsystems are present.
 
@@ -428,17 +454,18 @@ Note that `type` can only be either [`Operator`](@ref) or [`SuperOperator`](@ref
 !!! note
     `qeye` is a synonym of `eye`.
 """
-function eye(N::Int; type = Operator(), dims = nothing)
+function eye(::Type{T}, N::Int; type = Operator(), dims = nothing) where {T <: Number}
     if dims isa Nothing
         dims = isa(type, Operator) ? N : isqrt(N)
     end
-    return QuantumObject(Diagonal(ones(ComplexF64, N)); type = type, dims = dims)
+    return QuantumObject(Diagonal(ones(T, N)); type, dims)
 end
+eye(N::Int; type = Operator(), dims = nothing) = eye(ComplexF64, N; type, dims)
 
 @doc raw"""
-    fdestroy(N::Union{Int,Val}, j::Int)
+    fdestroy([T::Type=ComplexF64,] N::Union{Int,Val}, j::Int)
 
-Construct a fermionic destruction operator acting on the `j`-th site, where the fock space has totally `N`-sites:
+Construct a fermionic destruction operator [with element type `T = ComplexF64` (default)] acting on the `j`-th site, where the fock space has totally `N`-sites:
 
 Here, we use the [Jordan-Wigner transformation](https://en.wikipedia.org/wiki/Jordan%E2%80%93Wigner_transformation), namely
 ```math
@@ -452,12 +479,13 @@ Note that we put ``\hat{\sigma}_{+} = \begin{pmatrix} 0 & 1 \\ 0 & 0 \end{pmatri
 !!! warning "Beware of type-stability!"
     If you want to keep type stability, it is recommended to use `fdestroy(Val(N), j)` instead of `fdestroy(N, j)`. See [this link](https://docs.julialang.org/en/v1/manual/performance-tips/#man-performance-value-type) and the [related Section](@ref doc:Type-Stability) about type stability for more details.
 """
-fdestroy(N::Union{Int, Val}, j::Int) = _Jordan_Wigner(N, j, sigmap())
+fdestroy(::Type{T}, N::Union{Int, Val}, j::Int) where {T <: FloatOrComplex} = _Jordan_Wigner(T, N, j, sigmap(T))
+fdestroy(N::Union{Int, Val}, j::Int) = fdestroy(ComplexF64, N, j)
 
 @doc raw"""
-    fcreate(N::Union{Int,Val}, j::Int)
+    fcreate([T::Type=ComplexF64,] N::Union{Int,Val}, j::Int)
 
-Construct a fermionic creation operator acting on the `j`-th site, where the fock space has totally `N`-sites:
+Construct a fermionic creation operator [with element type `T = ComplexF64` (default)] acting on the `j`-th site, where the fock space has totally `N`-sites:
 
 Here, we use the [Jordan-Wigner transformation](https://en.wikipedia.org/wiki/Jordan%E2%80%93Wigner_transformation), namely
 ```math
@@ -471,39 +499,41 @@ Note that we put ``\hat{\sigma}_{-} = \begin{pmatrix} 0 & 0 \\ 1 & 0 \end{pmatri
 !!! warning "Beware of type-stability!"
     If you want to keep type stability, it is recommended to use `fcreate(Val(N), j)` instead of `fcreate(N, j)`. See [this link](https://docs.julialang.org/en/v1/manual/performance-tips/#man-performance-value-type) and the [related Section](@ref doc:Type-Stability) about type stability for more details.
 """
-fcreate(N::Union{Int, Val}, j::Int) = _Jordan_Wigner(N, j, sigmam())
+fcreate(::Type{T}, N::Union{Int, Val}, j::Int) where {T <: FloatOrComplex} = _Jordan_Wigner(T, N, j, sigmam(T))
+fcreate(N::Union{Int, Val}, j::Int) = fcreate(ComplexF64, N, j)
 
-_Jordan_Wigner(N::Int, j::Int, op::QuantumObject{Operator}) = _Jordan_Wigner(Val(N), j, op)
+_Jordan_Wigner(::Type{T}, N::Int, j::Int, op::QuantumObject{Operator}) where {T <: FloatOrComplex} = _Jordan_Wigner(T, Val(N), j, op)
 
-function _Jordan_Wigner(::Val{N}, j::Int, op::QuantumObject{Operator}) where {N}
+function _Jordan_Wigner(::Type{T}, ::Val{N}, j::Int, op::QuantumObject{Operator}) where {T <: FloatOrComplex, N}
     (N < 1) && throw(ArgumentError("The total number of sites (N) cannot be less than 1"))
     (1 <= j <= N) || throw(ArgumentError("The site index (j) should satisfy: 1 ≤ j ≤ N"))
 
-    σz = sigmaz().data
-    Z_tensor = kron(1, 1, fill(σz, j - 1)...)
+    σz = sigmaz(T).data
+    Z_tensor = kron(one(T), one(T), fill(σz, j - 1)...)
 
     S = 2^(N - j)
-    I_tensor = sparse((1.0 + 0.0im) * LinearAlgebra.I, S, S)
+    I_tensor = sparse(one(T) * LinearAlgebra.I, S, S)
 
     return QuantumObject(kron(Z_tensor, op.data, I_tensor); type = Operator(), dims = ntuple(i -> 2, Val(N)))
 end
 
 @doc raw"""
-    projection(N::Int, i::Int, j::Int)
+    projection([T::Type=ComplexF64,] N::Int, i::Int, j::Int)
 
-Generates the projection operator ``\hat{O} = |i \rangle\langle j|`` with Hilbert space dimension `N`.
+Generates the projection operator ``\hat{O} = |i \rangle\langle j|`` with Hilbert space dimension `N` and element type `T = ComplexF64` (default).
 """
-function projection(N::Int, i::Int, j::Int)
+function projection(::Type{T}, N::Int, i::Int, j::Int) where {T <: Number}
     (0 <= i < N) || throw(ArgumentError("Invalid argument i, must satisfy: 0 ≤ i ≤ N-1"))
     (0 <= j < N) || throw(ArgumentError("Invalid argument j, must satisfy: 0 ≤ j ≤ N-1"))
 
-    return QuantumObject(sparse([i + 1], [j + 1], [1.0 + 0.0im], N, N), type = Operator(), dims = N)
+    return QuantumObject(sparse([i + 1], [j + 1], [one(T)], N, N), type = Operator(), dims = N)
 end
+projection(N::Int, i::Int, j::Int) = projection(ComplexF64, N, i, j)
 
 @doc raw"""
-    tunneling(N::Int, m::Int=1; sparse::Union{Bool,Val{<:Bool}}=Val(false))
+    tunneling([T::Type=ComplexF64,] N::Int, m::Int=1; sparse::Union{Bool,Val{<:Bool}}=Val(false))
 
-Generate a tunneling operator defined as:
+Generate a tunneling operator with element type `T = ComplexF64` (default), defined as:
 
 ```math
 \sum_{n=0}^{N-m} | n \rangle\langle n+m | + | n+m \rangle\langle n |,
@@ -516,21 +546,22 @@ If `sparse=true`, the operator is returned as a sparse matrix, otherwise a dense
 !!! warning "Beware of type-stability!"
     If you want to keep type stability, it is recommended to use `tunneling(N, m, Val(sparse))` instead of `tunneling(N, m, sparse)`. See [this link](https://docs.julialang.org/en/v1/manual/performance-tips/#man-performance-value-type) and the [related Section](@ref doc:Type-Stability) about type stability for more details.
 """
-function tunneling(N::Int, m::Int = 1; sparse::Union{Bool, Val} = Val(false))
+function tunneling(::Type{T}, N::Int, m::Int = 1; sparse::Union{Bool, Val} = Val(false)) where {T <: Number}
     (m < 1) && throw(ArgumentError("The number of excitations (m) cannot be less than 1"))
 
-    data = ones(ComplexF64, N - m)
+    data = ones(T, N - m)
     if getVal(sparse)
         return QuantumObject(spdiagm(m => data, -m => data); type = Operator(), dims = N)
     else
         return QuantumObject(diagm(m => data, -m => data); type = Operator(), dims = N)
     end
 end
+tunneling(N::Int, m::Int = 1; sparse::Union{Bool, Val} = Val(false)) = tunneling(ComplexF64, N, m; sparse)
 
 @doc raw"""
-    qft(dimensions)
+    qft([T::Type=ComplexF64,] dimensions)
 
-Generates a discrete Fourier transform matrix ``\hat{F}_N`` for [Quantum Fourier Transform (QFT)](https://en.wikipedia.org/wiki/Quantum_Fourier_transform) with given argument `dimensions`.
+Generates a discrete Fourier transform matrix ``\hat{F}_N`` for [Quantum Fourier Transform (QFT)](https://en.wikipedia.org/wiki/Quantum_Fourier_transform) with given argument `dimensions` and element type `T = ComplexF64` (default).
 
 The `dimensions` can be either the following types:
 - `dimensions::Int`: Number of basis states in the Hilbert space.
@@ -554,12 +585,15 @@ where ``\omega = \exp(\frac{2 \pi i}{N})``.
 !!! warning "Beware of type-stability!"
     It is highly recommended to use `qft(dimensions)` with `dimensions` as `Tuple` or `SVector` from [StaticArrays.jl](https://github.com/JuliaArrays/StaticArrays.jl) to keep type stability. See the [related Section](@ref doc:Type-Stability) about type stability for more details.
 """
-qft(dimensions::Int) = QuantumObject(_qft_op(dimensions), Operator(), dimensions)
-qft(dimensions::Union{Dimensions, AbstractVector{Int}, Tuple}) =
-    QuantumObject(_qft_op(prod(dimensions)), Operator(), dimensions)
-function _qft_op(N::Int)
-    ω = exp(2.0im * π / N)
+qft(::Type{T}, dimensions::Int) where {T <: Complex} = QuantumObject(_qft_op(T, dimensions), Operator(), dimensions)
+qft(::Type{T}, dimensions::Union{Dimensions, AbstractVector{Int}, Tuple}) where {T <: Complex} =
+    QuantumObject(_qft_op(T, prod(dimensions)), Operator(), dimensions)
+qft(dimensions::Union{Int, Dimensions, AbstractVector{Int}, Tuple}) = qft(ComplexF64, dimensions)
+
+function _qft_op(::Type{T}, N::Int) where {T <: Complex}
+    N_T = T(N)
+    ω = exp(2 * im * T(π) / N_T)
     arr = 0:(N - 1)
     L, M = meshgrid(arr, arr)
-    return ω .^ (L .* M) / sqrt(N)
+    return ω .^ (L .* M) / sqrt(N_T)
 end
