@@ -2,13 +2,13 @@ export TimeEvolutionSol
 export TimeEvolutionMultiTrajSol, TimeEvolutionMCSol, TimeEvolutionStochasticSol
 export average_states, average_expect, std_expect
 
-export liouvillian_floquet, liouvillian_generalized
+export liouvillian_floquet, liouvillian_dressed_nonsecular
 
-const DEFAULT_ODE_SOLVER_OPTIONS = (abstol = 1e-8, reltol = 1e-6, save_everystep = false, save_end = true)
-const DEFAULT_SDE_SOLVER_OPTIONS = (abstol = 1e-3, reltol = 2e-3, save_everystep = false, save_end = true)
+const DEFAULT_ODE_SOLVER_OPTIONS = (abstol = 1.0e-8, reltol = 1.0e-6, save_everystep = false, save_end = true)
+const DEFAULT_SDE_SOLVER_OPTIONS = (abstol = 1.0e-3, reltol = 2.0e-3, save_everystep = false, save_end = true)
 const COL_TIMES_WHICH_INIT_SIZE = 200
 
-abstract type TimeEvolutionMultiTrajSol{Tstates,Texpect} end
+abstract type TimeEvolutionMultiTrajSol{Tstates, Texpect} end
 
 @doc raw"""
     struct TimeEvolutionProblem
@@ -27,12 +27,12 @@ A Julia constructor for handling the `ODEProblem` of the time evolution of quant
     For a given `prob::TimeEvolutionProblem`, `prob.dims` or `getproperty(prob, :dims)` returns its `dimensions` in the type of integer-vector.
 """
 struct TimeEvolutionProblem{
-    ST<:QuantumObjectType,
-    DT<:AbstractDimensions,
-    PT<:AbstractSciMLProblem,
-    TT<:AbstractVector,
-    KWT,
-}
+        ST <: QuantumObjectType,
+        DT <: AbstractDimensions,
+        PT <: AbstractSciMLProblem,
+        TT <: AbstractVector,
+        KWT,
+    }
     prob::PT
     times::TT
     states_type::ST
@@ -68,14 +68,14 @@ A structure storing the results and some information from solving time evolution
 - `reltol::Real`: The relative tolerance which is used during the solving process.
 """
 struct TimeEvolutionSol{
-    TT1<:AbstractVector{<:Real},
-    TT2<:AbstractVector{<:Real},
-    TS<:AbstractVector,
-    TE<:Union{AbstractMatrix,Nothing},
-    RETT<:Enum,
-    AlgT<:AbstractODEAlgorithm,
-    TolT<:Real,
-}
+        TT1 <: AbstractVector{<:Real},
+        TT2 <: AbstractVector{<:Real},
+        TS <: AbstractVector,
+        TE <: Union{AbstractMatrix, Nothing},
+        RETT <: Enum,
+        AlgT <: AbstractODEAlgorithm,
+        TolT <: Real,
+    }
     times::TT1
     times_states::TT2
     states::TS
@@ -140,15 +140,15 @@ We also provide the following functions for statistical analysis of multi-trajec
 - [`std_expect`](@ref)
 """
 struct TimeEvolutionMCSol{
-    TT1<:AbstractVector{<:Real},
-    TT2<:AbstractVector{<:Real},
-    TS<:AbstractVecOrMat,
-    TE<:Union{AbstractArray,Nothing},
-    TJT<:Vector{<:Vector{<:Real}},
-    TJW<:Vector{<:Vector{<:Integer}},
-    AlgT<:AbstractODEAlgorithm,
-    TolT<:Real,
-} <: TimeEvolutionMultiTrajSol{TS,TE}
+        TT1 <: AbstractVector{<:Real},
+        TT2 <: AbstractVector{<:Real},
+        TS <: AbstractVecOrMat,
+        TE <: Union{AbstractArray, Nothing},
+        TJT <: Vector{<:Vector{<:Real}},
+        TJW <: Vector{<:Vector{<:Integer}},
+        AlgT <: AbstractODEAlgorithm,
+        TolT <: Real,
+    } <: TimeEvolutionMultiTrajSol{TS, TE}
     ntraj::Int
     times::TT1
     times_states::TT2
@@ -215,14 +215,14 @@ We also provide the following functions for statistical analysis of multi-trajec
 - [`std_expect`](@ref)
 """
 struct TimeEvolutionStochasticSol{
-    TT1<:AbstractVector{<:Real},
-    TT2<:AbstractVector{<:Real},
-    TS<:AbstractVecOrMat,
-    TE<:Union{AbstractArray,Nothing},
-    TEM<:Union{AbstractArray,Nothing},
-    AlgT<:AbstractSDEAlgorithm,
-    TolT<:Real,
-} <: TimeEvolutionMultiTrajSol{TS,TE}
+        TT1 <: AbstractVector{<:Real},
+        TT2 <: AbstractVector{<:Real},
+        TS <: AbstractVecOrMat,
+        TE <: Union{AbstractArray, Nothing},
+        TEM <: Union{AbstractArray, Nothing},
+        AlgT <: AbstractSDEAlgorithm,
+        TolT <: Real,
+    } <: TimeEvolutionMultiTrajSol{TS, TE}
     ntraj::Int
     times::TT1
     times_states::TT2
@@ -263,7 +263,7 @@ average_states(sol::TimeEvolutionMultiTrajSol{<:Vector{<:QuantumObject}}) = sol.
 # TODO: Check if broadcasting division ./ size(states, 1) is type stable
 _average_traj_states(states::Matrix{<:QuantumObject{Ket}}) =
     map(x -> x / size(states, 1), dropdims(sum(ket2dm, states, dims = 1), dims = 1))
-_average_traj_states(states::Matrix{<:QuantumObject{ObjType}}) where {ObjType<:Union{Operator,OperatorKet}} =
+_average_traj_states(states::Matrix{<:QuantumObject{ObjType}}) where {ObjType <: Union{Operator, OperatorKet}} =
     map(x -> x / size(states, 1), dropdims(sum(states, dims = 1), dims = 1))
 
 @doc raw"""
@@ -271,19 +271,19 @@ _average_traj_states(states::Matrix{<:QuantumObject{ObjType}}) where {ObjType<:U
 
 Return the trajectory-averaged expectation values at each time point.
 """
-average_expect(sol::TimeEvolutionMultiTrajSol{TS,Array{T,3}}) where {TS,T<:Number} = _average_traj_expect(sol.expect)
-average_expect(sol::TimeEvolutionMultiTrajSol{TS,Matrix{T}}) where {TS,T<:Number} = sol.expect  # this case should already be averaged over all trajectories
-average_expect(::TimeEvolutionMultiTrajSol{TS,Nothing}) where {TS} = nothing
+average_expect(sol::TimeEvolutionMultiTrajSol{TS, Array{T, 3}}) where {TS, T <: Number} = _average_traj_expect(sol.expect)
+average_expect(sol::TimeEvolutionMultiTrajSol{TS, Matrix{T}}) where {TS, T <: Number} = sol.expect  # this case should already be averaged over all trajectories
+average_expect(::TimeEvolutionMultiTrajSol{TS, Nothing}) where {TS} = nothing
 
-_average_traj_expect(expvals::Array{T,3}) where {T<:Number} =
+_average_traj_expect(expvals::Array{T, 3}) where {T <: Number} =
     dropdims(sum(expvals, dims = 2), dims = 2) ./ size(expvals, 2)
 
 # these are used in multi-trajectory solvers before returning solutions
 _store_multitraj_states(states::Matrix{<:QuantumObject}, keep_runs_results::Val{false}) = _average_traj_states(states)
 _store_multitraj_states(states::Matrix{<:QuantumObject}, keep_runs_results::Val{true}) = states
-_store_multitraj_expect(expvals::Array{T,3}, keep_runs_results::Val{false}) where {T<:Number} =
+_store_multitraj_expect(expvals::Array{T, 3}, keep_runs_results::Val{false}) where {T <: Number} =
     _average_traj_expect(expvals)
-_store_multitraj_expect(expvals::Array{T,3}, keep_runs_results::Val{true}) where {T<:Number} = expvals
+_store_multitraj_expect(expvals::Array{T, 3}, keep_runs_results::Val{true}) where {T <: Number} = expvals
 _store_multitraj_expect(expvals::Nothing, keep_runs_results) = nothing
 
 @doc raw"""
@@ -291,7 +291,7 @@ _store_multitraj_expect(expvals::Nothing, keep_runs_results) = nothing
 
 Return the trajectory-wise standard deviation of the expectation values at each time point.
 """
-function std_expect(sol::TimeEvolutionMultiTrajSol{TS,Array{T,3}}) where {TS,T<:Number}
+function std_expect(sol::TimeEvolutionMultiTrajSol{TS, Array{T, 3}}) where {TS, T <: Number}
     # the following standard deviation (std) is defined as the square-root of variance instead of pseudo-variance
     # i.e., it is equivalent to (even for complex expectation values):
     #    dropdims(
@@ -301,12 +301,12 @@ function std_expect(sol::TimeEvolutionMultiTrajSol{TS,Array{T,3}}) where {TS,T<:
     # [this should be included in the runtest]
     return dropdims(std(sol.expect, corrected = false, dims = 2), dims = 2)
 end
-std_expect(::TimeEvolutionMultiTrajSol{TS,Matrix{T}}) where {TS,T<:Number} = throw(
+std_expect(::TimeEvolutionMultiTrajSol{TS, Matrix{T}}) where {TS, T <: Number} = throw(
     ArgumentError(
         "Can not compute the standard deviation without the expectation values of each trajectory. Try to specify keyword argument `keep_runs_results=Val(true)` to the solver.",
     ),
 )
-std_expect(::TimeEvolutionMultiTrajSol{TS,Nothing}) where {TS} = nothing
+std_expect(::TimeEvolutionMultiTrajSol{TS, Nothing}) where {TS} = nothing
 
 #######################################
 #=
@@ -375,12 +375,12 @@ function _ensemble_output_func_distributed(sol, i, channel, output_func)
 end
 
 function _ensemble_dispatch_output_func(
-    ::ET,
-    progress_bar,
-    ntraj,
-    output_func;
-    progr_desc = "Progress: ",
-) where {ET<:Union{EnsembleSerial,EnsembleThreads}}
+        ::ET,
+        progress_bar,
+        ntraj,
+        output_func;
+        progr_desc = "Progress: ",
+    ) where {ET <: Union{EnsembleSerial, EnsembleThreads}}
     if getVal(progress_bar)
         progr = Progress(ntraj; enabled = getVal(progress_bar), desc = progr_desc, settings.ProgressMeterKWARGS...)
         f = (sol, i) -> _ensemble_output_func_progress(sol, i, progr, output_func)
@@ -390,12 +390,12 @@ function _ensemble_dispatch_output_func(
     end
 end
 function _ensemble_dispatch_output_func(
-    ::ET,
-    progress_bar,
-    ntraj,
-    output_func;
-    progr_desc = "Progress... ",
-) where {ET<:Union{EnsembleSplitThreads,EnsembleDistributed}}
+        ::ET,
+        progress_bar,
+        ntraj,
+        output_func;
+        progr_desc = "Progress... ",
+    ) where {ET <: Union{EnsembleSplitThreads, EnsembleDistributed}}
     if getVal(progress_bar)
         progr = Progress(ntraj; enabled = getVal(progress_bar), desc = progr_desc, settings.ProgressMeterKWARGS...)
         progr_channel::RemoteChannel{Channel{Bool}} = RemoteChannel(() -> Channel{Bool}(1))
@@ -413,11 +413,11 @@ function _ensemble_dispatch_prob_func(rng, ntraj, tlist, prob_func; kwargs...)
 end
 
 function _ensemble_dispatch_solve(
-    ens_prob::TimeEvolutionProblem,
-    alg::Union{<:AbstractODEAlgorithm,<:AbstractSDEAlgorithm},
-    ensemblealg::ET,
-    ntraj::Int,
-) where {ET<:Union{EnsembleSplitThreads,EnsembleDistributed}}
+        ens_prob::TimeEvolutionProblem,
+        alg::Union{<:AbstractODEAlgorithm, <:AbstractSDEAlgorithm},
+        ensemblealg::ET,
+        ntraj::Int,
+    ) where {ET <: Union{EnsembleSplitThreads, EnsembleDistributed}}
     sol = nothing
 
     @sync begin
@@ -434,11 +434,11 @@ function _ensemble_dispatch_solve(
     return sol
 end
 function _ensemble_dispatch_solve(
-    ens_prob::TimeEvolutionProblem,
-    alg::Union{<:AbstractODEAlgorithm,<:AbstractSDEAlgorithm},
-    ensemblealg,
-    ntraj::Int,
-)
+        ens_prob::TimeEvolutionProblem,
+        alg::Union{<:AbstractODEAlgorithm, <:AbstractSDEAlgorithm},
+        ensemblealg,
+        ntraj::Int,
+    )
     sol = solve(ens_prob.prob, alg, ensemblealg, trajectories = ntraj)
     return sol
 end
@@ -500,18 +500,18 @@ end
 
 A struct to represent the diffusion operator. This is used to perform the diffusion process on N different Wiener processes.
 =#
-struct DiffusionOperator{T,OpType<:Tuple{Vararg{AbstractSciMLOperator}}}
+struct DiffusionOperator{T, OpType <: Tuple{Vararg{AbstractSciMLOperator}}}
     ops::OpType
     function DiffusionOperator(ops::OpType) where {OpType}
         T = mapreduce(eltype, promote_type, ops)
-        return new{T,OpType}(ops)
+        return new{T, OpType}(ops)
     end
 end
 
 @generated function (L::DiffusionOperator)(w, v, p, t)
     ops_types = L.parameters[2].parameters
     N = length(ops_types)
-    quote
+    return quote
         M = length(v)
         S = (size(w, 1), size(w, 2)) # This supports also `w` as a `Vector`
         (S[1] == M && S[2] == $N) || throw(DimensionMismatch("The size of the output vector is incorrect."))
@@ -526,82 +526,93 @@ end
 #######################################
 
 function liouvillian_floquet(
-    L₀::QuantumObject{SuperOperator},
-    Lₚ::QuantumObject{SuperOperator},
-    Lₘ::QuantumObject{SuperOperator},
-    ω::Real;
-    n_max::Int = 3,
-    tol::Real = 1e-15,
-)
+        L₀::QuantumObject{SuperOperator},
+        Lₚ::QuantumObject{SuperOperator},
+        Lₘ::QuantumObject{SuperOperator},
+        ω::Real;
+        n_max::Int = 3,
+        tol::Real = 1.0e-15,
+    )
     check_dimensions(L₀, Lₚ, Lₘ)
     return _liouvillian_floquet(L₀, Lₚ, Lₘ, ω, n_max, tol)
 end
 
 function liouvillian_floquet(
-    H::QuantumObject{OpType1},
-    Hₚ::QuantumObject{OpType2},
-    Hₘ::QuantumObject{OpType3},
-    ω::Real,
-    c_ops::Union{Nothing,AbstractVector,Tuple} = nothing;
-    n_max::Int = 3,
-    tol::Real = 1e-15,
-) where {
-    OpType1<:Union{Operator,SuperOperator},
-    OpType2<:Union{Operator,SuperOperator},
-    OpType3<:Union{Operator,SuperOperator},
-}
+        H::QuantumObject{OpType1},
+        Hₚ::QuantumObject{OpType2},
+        Hₘ::QuantumObject{OpType3},
+        ω::Real,
+        c_ops::Union{Nothing, AbstractVector, Tuple} = nothing;
+        n_max::Int = 3,
+        tol::Real = 1.0e-15,
+    ) where {
+        OpType1 <: Union{Operator, SuperOperator},
+        OpType2 <: Union{Operator, SuperOperator},
+        OpType3 <: Union{Operator, SuperOperator},
+    }
     return liouvillian_floquet(liouvillian(H, c_ops), liouvillian(Hₚ), liouvillian(Hₘ), ω, n_max = n_max, tol = tol)
 end
 
 @doc raw"""
-    liouvillian_generalized(H::QuantumObject, fields::Vector, 
-    T_list::Vector; N_trunc::Int=size(H,1), tol::Float64=0.0, σ_filter::Union{Nothing, Real}=nothing)
+    liouvillian_dressed_nonsecular(
+        H::QuantumObject{Operator},
+        fields::Vector,
+        T_list::Vector{<:Real};
+        N_trunc::Union{Int,Nothing}=nothing,
+        tol::Real=1e-12,
+        σ_filter::Union{Nothing,Real}=nothing,
+    )
 
-Constructs the generalized Liouvillian for a system coupled to a bath of harmonic oscillators.
+Build the generalized Liouvillian for a system coupled to multiple bosonic baths in the ultrastrong-coupling regime. The Hamiltonian `H` is diagonalized, the system-bath operators in `fields` are projected into the eigenbasis, and thermal jump operators are assembled for each temperature in `T_list`.
 
-See, e.g., Settineri, Alessio, et al. "Dissipation and thermal noise in hybrid quantum systems in the ultrastrong-coupling regime." Physical Review A 98.5 (2018): 053834.
+# Arguments
+- `H::QuantumObject{Operator}`: System Hamiltonian.
+- `fields::Vector`: Coupling operators that mediate the interaction with each bath; must match the length of `T_list`.
+- `T_list::Vector{<:Real}`: Bath temperatures ordered consistently with `fields`.
+
+# Keyword Arguments
+- `N_trunc::Union{Int,Nothing}`: If provided, truncate the eigenbasis to the first `N_trunc` levels; otherwise use the full dimension of `H`.
+- `tol::Real`: Tolerance passed to sparsification utilities when constructing the filters and dissipators.
+- `σ_filter::Union{Nothing,Real}`: Width of the Gaussian frequency filter. If `nothing`, a heuristic value proportional to the coupling strengths is used.
+
+# Returns
+- `E`: Eigenenergies of `H` (truncated if `N_trunc` is provided).
+- `U`: Eigenvectors of `H` as a [`QuantumObject`](@ref) mapping to the truncated basis.
+- `L`: Generalized Liouvillian [`SuperOperator`](@ref) including the frequency-filtered dissipators.
+
+# References
+- [Settineri2018](@cite)
 """
-function liouvillian_generalized(
-    H::QuantumObject{Operator},
-    fields::Vector,
-    T_list::Vector{<:Real};
-    N_trunc::Union{Int,Nothing} = nothing,
-    tol::Real = 1e-12,
-    σ_filter::Union{Nothing,Real} = nothing,
-)
-    (length(fields) == length(T_list)) || throw(DimensionMismatch("The number of fields, ωs and Ts must be the same."))
+function liouvillian_dressed_nonsecular(
+        H::QuantumObject{Operator},
+        fields::Vector,
+        T_list::Vector{<:Real};
+        N_trunc::Union{Int, Nothing} = nothing,
+        tol::Real = 1.0e-12,
+        σ_filter::Union{Nothing, Real} = nothing,
+    )
+    (length(fields) == length(T_list)) || throw(DimensionMismatch("The number of fields and T_list must be the same."))
 
-    dims = (N_trunc isa Nothing) ? H.dimensions : SVector(N_trunc)
+    dims = isnothing(N_trunc) ? H.dims : (N_trunc,)
     final_size = prod(dims)
+    final_dims = isnothing(N_trunc) ? H.dims : (H.dims, dims)
     result = eigen(H)
     E = real.(result.values[1:final_size])
-    U = QuantumObject(result.vectors, result.type, result.dimensions)
+    U = QuantumObject(result.vectors[:, 1:final_size], result.type, final_dims)
 
     H_d = QuantumObject(Diagonal(complex(E)), type = Operator(), dims = dims)
 
     Ω = E' .- E
     Ωp = triu(to_sparse(Ω, tol), 1)
 
-    # Filter in the Hilbert space
+    # Filter width
     σ = isnothing(σ_filter) ? 500 * maximum([norm(field) / length(field) for field in fields]) : σ_filter
-    F1 = QuantumObject(gaussian.(Ω, 0, σ), type = Operator(), dims = dims)
-    F1 = to_sparse(F1, tol)
-
-    # Filter in the Liouville space
-    # M1 = ones(final_size, final_size)
-    M1 = similar(E, final_size, final_size)
-    M1 .= 1
-    Ω1 = kron(Ω, M1)
-    Ω2 = kron(M1, Ω)
-    Ωdiff = Ω1 .- Ω2
-    F2 = QuantumObject(gaussian.(Ωdiff, 0, σ), SuperOperator(), dims)
-    F2 = to_sparse(F2, tol)
 
     L = liouvillian(H_d)
 
     for i in eachindex(fields)
         # The operator that couples the system to the bath in the eigenbasis
-        X_op = to_sparse((U'*fields[i]*U).data[1:final_size, 1:final_size], tol)
+        X_op = to_sparse((U' * fields[i] * U).data, tol)
         if ishermitian(fields[i])
             X_op = (X_op + X_op') / 2 # Make sure it's hermitian
         end
@@ -613,25 +624,59 @@ function liouvillian_generalized(
         Sp₂ = QuantumObject(droptol!((@. Ωp * (1 + N_th) * Sp₀.data), tol), type = Operator(), dims = dims)
         # S0 = QuantumObject( spdiagm(diag(X_op)), dims=dims )
 
-        L +=
-            1 / 2 *
-            (F2 .* (sprepost(Sp₁', Sp₀) + sprepost(Sp₀', Sp₁)) - spre(F1 .* (Sp₀ * Sp₁')) - spost(F1 .* (Sp₁ * Sp₀')))
-        L +=
-            1 / 2 *
-            (F2 .* (sprepost(Sp₂, Sp₀') + sprepost(Sp₀, Sp₂')) - spre(F1 .* (Sp₀' * Sp₂)) - spost(F1 .* (Sp₂' * Sp₀)))
+        # Build the dissipator contribution and apply Gaussian filter in Liouville space
+        D₁ = 1 / 2 * (sprepost(Sp₁', Sp₀) + sprepost(Sp₀', Sp₁) - spre(Sp₀ * Sp₁') - spost(Sp₁ * Sp₀'))
+        D₂ = 1 / 2 * (sprepost(Sp₂, Sp₀') + sprepost(Sp₀, Sp₂') - spre(Sp₀' * Sp₂) - spost(Sp₂' * Sp₀))
+
+        L += _apply_liouville_filter(D₁ + D₂, E, σ, tol)
     end
 
+    settings.auto_tidyup && tidyup!(L)
+
     return E, U, L
 end
 
+function _apply_liouville_filter(
+        L::QuantumObject{SuperOperator, DimsType, MT},
+        E::AbstractVector,
+        σ::Real,
+        tol::Real,
+    ) where {DimsType, MT <: AbstractSparseMatrix}
+    data = L.data
+    N = length(E)
+
+    I, J, V = findnz(data)
+
+    # Broadcast the filter computation for GPU compatibility
+    V .*= _liouville_filter_weights.(I, J, N, Ref(E), σ)
+
+    # Reconstruct sparse matrix, filtering out small values
+    mask = abs.(V) .> tol
+    filtered_data = sparse(I[mask], J[mask], V[mask], size(data)...)
+    return QuantumObject(filtered_data, SuperOperator(), L.dimensions)
+end
+
+function _liouville_filter_weights(row::Int, col::Int, N::Int, E::AbstractVector, σ::Real)
+    j, l = _linear_to_subscript(row, N)
+    k, m = _linear_to_subscript(col, N)
+    Ωdiff = (E[j] - E[k]) - (E[l] - E[m])
+    return gaussian(Ωdiff, 0, σ)
+end
+
+function _linear_to_subscript(idx::Int, N::Int)
+    row = mod1(idx, N)
+    col = div(idx - 1, N) + 1
+    return row, col
+end
+
 function _liouvillian_floquet(
-    L₀::QuantumObject{SuperOperator},
-    Lₚ::QuantumObject{SuperOperator},
-    Lₘ::QuantumObject{SuperOperator},
-    ω::Real,
-    n_max::Int,
-    tol::Real,
-)
+        L₀::QuantumObject{SuperOperator},
+        Lₚ::QuantumObject{SuperOperator},
+        Lₘ::QuantumObject{SuperOperator},
+        ω::Real,
+        n_max::Int,
+        tol::Real,
+    )
     L_0 = L₀.data
     L_p = Lₚ.data
     L_m = Lₘ.data
@@ -641,7 +686,7 @@ function _liouvillian_floquet(
     S = -(L_0 - 1im * n_max * ω * I) \ L_p_dense
     T = -(L_0 + 1im * n_max * ω * I) \ L_m_dense
 
-    for n_i in (n_max-1):-1:1
+    for n_i in (n_max - 1):-1:1
         S = -(L_0 - 1im * n_i * ω * I + L_m * S) \ L_p_dense
         T = -(L_0 + 1im * n_i * ω * I + L_p * T) \ L_m_dense
     end
